@@ -3,6 +3,8 @@ Provides classes and utilities to allow you to interact with an instance of the
 platform.
 """
 
+from __future__ import annotations
+
 import errno
 import json
 import logging
@@ -223,7 +225,7 @@ class KodexaPlatform:
     """
 
     @staticmethod
-    def get_access_token() -> 'str':
+    def get_access_token() -> str:
         """
         Returns the access token
 
@@ -237,7 +239,7 @@ class KodexaPlatform:
         return access_token if access_token is not None else kodexa_config['access_token']
 
     @staticmethod
-    def get_url() -> 'str':
+    def get_url() -> str:
         """
         Returns the URL to use to access a Kodexa Platform
 
@@ -283,7 +285,7 @@ class KodexaPlatform:
             os.environ["KODEXA_URL"] = url
 
     @staticmethod
-    def get_access_token_details() -> 'Dict':
+    def get_access_token_details() -> Dict:
         """
         Pull the access token details (including a list of the available organizations)
 
@@ -836,16 +838,16 @@ class RemoteSession:
         if attach_source:
             logger.debug("Attaching source to call")
             files["file"] = get_source(document)
-            files["file_document"] = document.to_msgpack()
+            files["file_document"] = document.to_kddb()
         else:
-            files["document"] = document.to_msgpack()
+            files["document"] = document.to_kddb()
 
         data = {"options": json.dumps(options), "document_metadata_json": json.dumps(document.metadata),
                 "context": json.dumps(context.context)}
 
         logger.info(f"Executing session {self.cloud_session.id}")
         r = requests.post(f"{KodexaPlatform.get_url()}/api/sessions/{self.cloud_session.id}/execute",
-                          params={self.session_type: self.slug},
+                          params={self.session_type: self.slug, "documentVersion": document.version},
                           data=data,
                           headers={"x-access-token": KodexaPlatform.get_access_token()}, files=files)
         try:
@@ -1044,7 +1046,7 @@ class RemotePipeline:
         return self.context
 
     @staticmethod
-    def from_url(slug: str, url, headers=None, *args, **kwargs) -> 'RemotePipeline':
+    def from_url(slug: str, url, headers=None, *args, **kwargs) -> RemotePipeline:
         """Build a new pipeline with the input being a document created from the given URL
 
         Args:
@@ -1062,7 +1064,7 @@ class RemotePipeline:
         return RemotePipeline(slug, Document.from_url(url, headers), *args, **kwargs)
 
     @staticmethod
-    def from_file(slug: str, file_path: str, unpack: bool = False, *args, **kwargs) -> 'RemotePipeline':
+    def from_file(slug: str, file_path: str, unpack: bool = False, *args, **kwargs) -> RemotePipeline:
         """Create a new pipeline using a file path as a source
 
         Args:
@@ -1082,7 +1084,7 @@ class RemotePipeline:
         return RemotePipeline(slug, Document.from_file(file_path, unpack), *args, **kwargs)
 
     @staticmethod
-    def from_text(slug: str, text: str, *args, **kwargs) -> 'RemotePipeline':
+    def from_text(slug: str, text: str, *args, **kwargs) -> RemotePipeline:
         """Build a new pipeline and provide text as the basic to create a document
 
         Args:
@@ -1106,7 +1108,7 @@ class RemotePipeline:
     def from_folder(slug: str, folder_path: str, filename_filter: str = "*", recursive: bool = False,
                     unpack: bool = False,
                     relative: bool = False,
-                    caller_path: str = get_caller_dir()) -> 'RemotePipeline':
+                    caller_path: str = get_caller_dir()) -> RemotePipeline:
         """Create a pipeline that will run against a set of local files from a folder
 
         Args:
