@@ -1,7 +1,4 @@
 """
-Mix-ins
--------
-
 Mix-ins are an effective way to add helper functionality to Documents and ContentNode's based on the underlying
 features.
 """
